@@ -9,8 +9,7 @@ import sys
 import time
 
 from dns.resolver import query
-from lxml.html import document_fromstring
-from lxml.html.html5parser import document_fromstring as html5_fromstring
+from lxml.html.html5parser import document_fromstring
 from lxml.etree import fromstring, tostring
 from requests import post, request
 from waitress import serve
@@ -29,7 +28,7 @@ API_IP_ADDRESS = query(API_HOSTNAME)[0].to_text()
 HOPPISH_HEADERS = {
     'connection', 'keep-alive', 'proxy-authenticate',
     'proxy-authorization', 'te', 'trailers', 'transfer-encoding',
-    'upgrade'
+    'upgrade', 'content-encoding'
 }
 
 
@@ -84,7 +83,7 @@ def translate_response(xml, language):
     return tostring(document, encoding='utf-8')
 
 
-TRANSLATE_API_URL = 'http://translate.google.hu/translate_t'
+TRANSLATE_API_URL = 'http://translate.google.co.jp/translate_t'
 
 
 def translate(text, language):
@@ -99,75 +98,39 @@ def translate(text, language):
         data={
             'text': text,
             'hl': language,
-            'langpair': 'auto|' + language,
+            'langpair': 'ja|' + language,
             'tbb': '1'
         }
     )
     document = document_fromstring(response.content)
-    try:
-        result = document.get_element_by_id('result_box')
-    except (KeyError, IndexError):
-        document = html5_fromstring(response.content)
-        result = document.xpath('descendant-or-self::*[@id="result_box"]')
-        return result[0].xpath('string()')
-    else:
-        return result.text_content()
+    result = document.xpath('descendant-or-self::*[@id="result_box"]')
+    return result[0].xpath('string()')
 
 
-LIMITS = [30000, 10000, 5000]
+BUFFER_LIMIT = 20000
 
 
-def translate_multiple(text_list, language, limit=30000):
+def translate_multiple(text_list, language):
     """Translate many texts at a time."""
+    text_list = list(text_list)
     boundary = ' {0} '.format(int(time.time() * random.randrange(1000, 9999)))
-    boundary_size = len(boundary)
-    buffer = []
-    buffer_size = 0
-    input_size = 0
+    total = len(text_list)
     result_list = []
-    def _flush_buffer():
-        nonlocal buffer, buffer_size, input_size, result_list
-        if buffer_size:
-            result = translate(''.join(buffer), language)
-            sublist = re.split(r'\s*' + boundary.strip() + r'\s*', result)
-            sublist_size = len(sublist)
-            assert sublist_size == input_size, \
-                   'expected {0}, but got {1}'.format(input_size, sublist_size)
-            result_list += sublist
-            buffer = []
-            buffer_size = 0
-            input_size = 0
-    def flush_buffer():
-        try:
-            _flush_buffer()
-        except AssertionError:
-            try:
-                _flush_buffer()  # retry
-            except AssertionError:
-                return False
-        return True
-    def retry():
-        """Retry translation with more thight limit."""
-        try:
-            next_limit = LIMITS[LIMITS.index(limit) + 1]
-        except (IndexError, ValueError):
-            assert limit > 1
-            next_limit = limit / 2
-        return translate_multiple(text_list, language, next_limit)
-    for text in text_list:
-        text_size = len(text)
-        added_size = buffer_size + (buffer_size and boundary_size) + text_size
-        if added_size > limit:
-            if not flush_buffer():
-                return retry()
-            added_size = text_size
-        if buffer_size:
-            buffer.append(boundary)
-        buffer.append(text)
-        input_size += 1
-        buffer_size = added_size
-    if not flush_buffer():
-        return retry()
+    while len(result_list) < total:
+        j = len(result_list)
+        input_list = []
+        buffer_limit = BUFFER_LIMIT
+        while buffer_limit > 0:
+            while j < total and sum(map(len, input_list)) < buffer_limit:
+                input_list.append(text_list[j])
+                input_list.append(boundary)
+                j += 1
+            translated = translate(''.join(input_list), language)
+            split = re.split(r'\s*' + boundary.strip() + '\s*', translated)[:-1]
+            if split:
+                break
+            buffer_limit //= 2
+        result_list += split
     return result_list
 
 
